// WaveIOThread.cpp: implementation of the CWaveIOThread class.
//
// This source file is a supplement to the article "Digitalni zvuk". The article
// was published in the #34 issue of the "PC" computer monthly, published in
// Belgrade, Yugoslavia by "PC Press"
//
// You can freely use, modify and redistribute this file, provided that this
// header is kept unchanged
//
//                                                                 Ivica Nikolic
//                                                      ivica@galeb.etf.bg.ac.yu
//
////////////////////////////////////////////////////////////////////////////////

#include "StdInclude.h"
#include "WaveIOThread.h"

CWaveIOThread::CWaveIOThread(
	WAVEFORMATEX* pwfmt,
	UINT nBufferSize,
	UINT nIOBuffers,
	UINT nRingBuffers
) :
	m_buff( nBufferSize, nRingBuffers )
{
	memcpy( &m_wfmt, pwfmt, sizeof(m_wfmt) );
	m_nBufferSize = nBufferSize;
	m_nIOBuffers = nIOBuffers;
	m_nRingBuffers = nRingBuffers;

	m_pwhdrIOBuffers = NULL;
	m_hEventIOCompletion = NULL;

	if ( GE_INST.HasOccured() )
		return;

	m_pwhdrIOBuffers = (WAVEHDR*)GlobalAllocPtr( GHND,
		m_nIOBuffers * sizeof(*m_pwhdrIOBuffers) );
	for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
	{
		m_pwhdrIOBuffers[ nBuffer ].lpData = (LPSTR)GlobalAllocPtr( GHND, m_nBufferSize );
		m_pwhdrIOBuffers[ nBuffer ].dwBufferLength = m_nBufferSize;
	}

	m_hEventIOCompletion = CreateEvent( NULL, TRUE, FALSE, NULL );

	m_hEventsForWait[ 0 ] = GE_INST.GetHandle();
	m_hEventsForWait[ 1 ] = m_hEventShouldFinish;
	m_hEventsForWait[ 2 ] = m_hEventIOCompletion;
}

CWaveIOThread::~CWaveIOThread()
{
	if ( m_hEventIOCompletion!=NULL )
		CloseHandle( m_hEventIOCompletion );

	if ( m_pwhdrIOBuffers!=NULL )
	{
		for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
			GlobalFreePtr( m_pwhdrIOBuffers[nBuffer].lpData );
		GlobalFreePtr( m_pwhdrIOBuffers );
	}
}

CRingBuffer* CWaveIOThread::GetBuff()
{
	return &m_buff;
}

UINT CWaveIOThread::WaitForEvent( BOOL& bShouldResetEvent )
{
	bShouldResetEvent = TRUE;

	if ( WaitForSingleObject(m_hEventIOCompletion, 0)==WAIT_TIMEOUT )
	{
		for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
			if ( (m_pwhdrIOBuffers[nBuffer].dwFlags & WHDR_DONE) != 0 )
			{
				bShouldResetEvent = FALSE;
				return nBuffer;
			}
	}

	DWORD dwEvent = WaitForMultipleObjects( 3, m_hEventsForWait,
		FALSE, INFINITE );

// return -1 if global error is raised or thread should finish
	if ( dwEvent==WAIT_OBJECT_0 || dwEvent==WAIT_OBJECT_0+1 )
		return -1;

	ResetEvent( m_hEventIOCompletion );

	for ( UINT nBuffer=0; nBuffer<m_nIOBuffers; nBuffer++ )
		if ( (m_pwhdrIOBuffers[nBuffer].dwFlags & WHDR_DONE) != 0 )
			return nBuffer;

// if something went wrong, return -1 and raise global error
	GE_INST.Raise();
	return -1;
}
